using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using JewelryStoreContracts.BindingModels;
using JewelryStoreContracts.SearchModels;
using JewelryStoreContracts.ViewModels;


namespace JewelryStoreContracts.StoragesContracts
{
    public interface IJewelStorage
    {
        List<JewelViewModel> GetFullList();
        List<JewelViewModel> GetFilteredList(JewelSearchModel model);
        JewelViewModel? GetElement(JewelSearchModel model);
        JewelViewModel? Insert(JewelBindingModel model);
        JewelViewModel? Update(JewelBindingModel model);
        JewelViewModel? Delete(JewelBindingModel model);
    }

}
